@@ -1,6 +1,6 @@
 //
 // VMime library (http://www.vmime.org)
-// Copyright (C) 2002-2005 Vincent Richard <vincent@vincent-richard.net>
+// Copyright (C) 2002-2006 Vincent Richard <vincent@vincent-richard.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -46,6 +46,12 @@ exception::exception(const string& what, const exception& other)
 }
 
 
+exception::exception(const exception& e)
+	: std::exception(), m_what(e.what()), m_other(e.m_other == NULL ? NULL : e.m_other->clone())
+{
+}
+
+
 exception::~exception() throw()
 {
 	delete (m_other);
@@ -117,8 +123,8 @@ const char* charset_conv_error::name() const throw() { return "charset_conv_erro
 //
 
 no_encoder_available::~no_encoder_available() throw() {}
-no_encoder_available::no_encoder_available(const exception& other)
-	: exception("No encoder available.", other) {}
+no_encoder_available::no_encoder_available(const string& name, const exception& other)
+	: exception("No encoder available: '" + name + "'.", other) {}
 
 exception* no_encoder_available::clone() const { return new no_encoder_available(*this); }
 const char* no_encoder_available::name() const throw() { return "no_encoder_available"; }
@@ -233,15 +239,15 @@ const char* no_factory_available::name() const throw() { return "no_factory_avai
 
 
 //
-// no_platform_dependant_handler
+// no_platform_handler
 //
 
-no_platform_dependant_handler::~no_platform_dependant_handler() throw() {}
-no_platform_dependant_handler::no_platform_dependant_handler(const exception& other)
-	: exception("No platform-dependant handler installed.", other) {}
+no_platform_handler::~no_platform_handler() throw() {}
+no_platform_handler::no_platform_handler(const exception& other)
+	: exception("No platform handler installed.", other) {}
 
-exception* no_platform_dependant_handler::clone() const { return new no_platform_dependant_handler(*this); }
-const char* no_platform_dependant_handler::name() const throw() { return "no_platform_dependant_handler"; }
+exception* no_platform_handler::clone() const { return new no_platform_handler(*this); }
+const char* no_platform_handler::name() const throw() { return "no_platform_handler"; }
 
 
 //
