@@ -1,6 +1,6 @@
 //
 // VMime library (http://www.vmime.org)
-// Copyright (C) 2002-2005 Vincent Richard <vincent@vincent-richard.net>
+// Copyright (C) 2002-2006 Vincent Richard <vincent@vincent-richard.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -28,7 +28,7 @@
 #include "vmime/message.hpp"
 
 #include "vmime/exception.hpp"
-#include "vmime/platformDependant.hpp"
+#include "vmime/platform.hpp"
 
 
 namespace vmime {
@@ -36,23 +36,25 @@ namespace net {
 namespace maildir {
 
 
-maildirFolder::maildirFolder(const folder::path& path, weak_ref <maildirStore> store)
+maildirFolder::maildirFolder(const folder::path& path, ref <maildirStore> store)
 	: m_store(store), m_path(path),
 	  m_name(path.isEmpty() ? folder::path::component("") : path.getLastComponent()),
 	  m_mode(-1), m_open(false), m_unreadMessageCount(0), m_messageCount(0)
 {
-	m_store->registerFolder(this);
+	store->registerFolder(this);
 }
 
 
 maildirFolder::~maildirFolder()
 {
-	if (m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (store)
 	{
 		if (m_open)
 			close(false);
 
-		m_store->unregisterFolder(this);
+		store->unregisterFolder(this);
 	}
 	else if (m_open)
 	{
@@ -89,10 +91,10 @@ const int maildirFolder::getFlags()
 {
 	int flags = 0;
 
-	utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+	utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 	ref <utility::file> rootDir = fsf->create
-		(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_CONTAINER));
+		(maildirUtils::getFolderFSPath(m_store.acquire(), m_path, maildirUtils::FOLDER_PATH_CONTAINER));
 
 	ref <utility::fileIterator> it = rootDir->getFiles();
 
@@ -125,7 +127,9 @@ const folder::path maildirFolder::getFullPath() const
 
 void maildirFolder::open(const int mode, bool /* failIfModeIsNotAvailable */)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (isOpen())
 		throw exceptions::illegal_state("Folder is already open");
@@ -141,7 +145,9 @@ void maildirFolder::open(const int mode, bool /* failIfModeIsNotAvailable */)
 
 void maildirFolder::close(const bool expunge)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 
 	if (!isOpen())
@@ -187,32 +193,34 @@ void maildirFolder::unregisterMessage(maildirMessage* msg)
 
 void maildirFolder::create(const int /* type */)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (isOpen())
 		throw exceptions::illegal_state("Folder is open");
 	else if (exists())
 		throw exceptions::illegal_state("Folder already exists");
-	else if (!m_store->isValidFolderName(m_name))
+	else if (!store->isValidFolderName(m_name))
 		throw exceptions::invalid_folder_name();
 
 	// Create directory on file system
 	try
 	{
-		utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+		utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
-		if (!fsf->isValidPath(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_ROOT)))
+		if (!fsf->isValidPath(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_ROOT)))
 			throw exceptions::invalid_folder_name();
 
 		ref <utility::file> rootDir = fsf->create
-			(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_ROOT));
+			(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_ROOT));
 
 		ref <utility::file> newDir = fsf->create
-			(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_NEW));
+			(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_NEW));
 		ref <utility::file> tmpDir = fsf->create
-			(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_TMP));
+			(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_TMP));
 		ref <utility::file> curDir = fsf->create
-			(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_CUR));
+			(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_CUR));
 
 		rootDir->createDirectory(true);
 
@@ -234,19 +242,57 @@ void maildirFolder::create(const int /* type */)
 }
 
 
-const bool maildirFolder::exists()
+void maildirFolder::destroy()
 {
-	utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
+		throw exceptions::illegal_state("Store disconnected");
+	else if (isOpen())
+		throw exceptions::illegal_state("Folder is open");
+
+	// Delete 'folder' and '.folder.directory' directories
+	utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 	ref <utility::file> rootDir = fsf->create
-		(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_ROOT));
+		(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_ROOT));
+	ref <utility::file> contDir = fsf->create
+		(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_CONTAINER));
+
+	try
+	{
+		maildirUtils::recursiveFSDelete(rootDir);
+		maildirUtils::recursiveFSDelete(contDir);
+	}
+	catch (std::exception&)
+	{
+		// Ignore exception: anyway, we can't recover from this...
+	}
+
+	// Notify folder deleted
+	events::folderEvent event
+		(thisRef().dynamicCast <folder>(),
+		 events::folderEvent::TYPE_DELETED, m_path, m_path);
+
+	notifyFolder(event);
+}
+
+
+const bool maildirFolder::exists()
+{
+	ref <maildirStore> store = m_store.acquire();
+
+	utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
+
+	ref <utility::file> rootDir = fsf->create
+		(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_ROOT));
 
 	ref <utility::file> newDir = fsf->create
-		(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_NEW));
+		(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_NEW));
 	ref <utility::file> tmpDir = fsf->create
-		(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_TMP));
+		(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_TMP));
 	ref <utility::file> curDir = fsf->create
-		(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_CUR));
+		(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_CUR));
 
 	return (rootDir->exists() && rootDir->isDirectory() &&
 	        newDir->exists() && newDir->isDirectory() &&
@@ -263,19 +309,21 @@ const bool maildirFolder::isOpen() const
 
 void maildirFolder::scanFolder()
 {
+	ref <maildirStore> store = m_store.acquire();
+
 	try
 	{
 		m_messageCount = 0;
 		m_unreadMessageCount = 0;
 
-		utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+		utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 		utility::file::path newDirPath = maildirUtils::getFolderFSPath
-			(m_store, m_path, maildirUtils::FOLDER_PATH_NEW);
+			(store, m_path, maildirUtils::FOLDER_PATH_NEW);
 		ref <utility::file> newDir = fsf->create(newDirPath);
 
 		utility::file::path curDirPath = maildirUtils::getFolderFSPath
-			(m_store, m_path, maildirUtils::FOLDER_PATH_CUR);
+			(store, m_path, maildirUtils::FOLDER_PATH_CUR);
 		ref <utility::file> curDir = fsf->create(curDirPath);
 
 		// New received messages (new/)
@@ -352,7 +400,11 @@ void maildirFolder::scanFolder()
 			// Append to message list
 			messageInfos msgInfos;
 			msgInfos.path = newFilename;
-			msgInfos.type = messageInfos::TYPE_CUR;
+
+			if (maildirUtils::extractFlags(msgInfos.path) & message::FLAG_DELETED)
+				msgInfos.type = messageInfos::TYPE_DELETED;
+			else
+				msgInfos.type = messageInfos::TYPE_CUR;
 
 			m_messageInfos.push_back(msgInfos);
 		}
@@ -365,7 +417,11 @@ void maildirFolder::scanFolder()
 			// Append to message list
 			messageInfos msgInfos;
 			msgInfos.path = *it;
-			msgInfos.type = messageInfos::TYPE_CUR;
+
+			if (maildirUtils::extractFlags(msgInfos.path) & message::FLAG_DELETED)
+				msgInfos.type = messageInfos::TYPE_DELETED;
+			else
+				msgInfos.type = messageInfos::TYPE_CUR;
 
 			m_messageInfos.push_back(msgInfos);
 		}
@@ -399,7 +455,7 @@ ref <message> maildirFolder::getMessage(const int num)
 		throw exceptions::message_not_found();
 
 	return vmime::create <maildirMessage>
-		(thisWeakRef().dynamicCast <maildirFolder>(), num);
+		(thisRef().dynamicCast <maildirFolder>(), num);
 }
 
 
@@ -413,12 +469,10 @@ std::vector <ref <message> > maildirFolder::getMessages(const int from, const in
 		throw exceptions::message_not_found();
 
 	std::vector <ref <message> > v;
+	ref <maildirFolder> thisFolder = thisRef().dynamicCast <maildirFolder>();
 
 	for (int i = from ; i <= to2 ; ++i)
-	{
-		v.push_back(vmime::create <maildirMessage>
-			(thisWeakRef().dynamicCast <maildirFolder>(), i));
-	}
+		v.push_back(vmime::create <maildirMessage>(thisFolder, i));
 
 	return (v);
 }
@@ -430,12 +484,10 @@ std::vector <ref <message> > maildirFolder::getMessages(const std::vector <int>&
 		throw exceptions::illegal_state("Folder not open");
 
 	std::vector <ref <message> > v;
+	ref <maildirFolder> thisFolder = thisRef().dynamicCast <maildirFolder>();
 
 	for (std::vector <int>::const_iterator it = nums.begin() ; it != nums.end() ; ++it)
-	{
-		v.push_back(vmime::create <maildirMessage>
-			(thisWeakRef().dynamicCast <maildirFolder>(), *it));
-	}
+		v.push_back(vmime::create <maildirMessage>(thisFolder, *it));
 
 	return (v);
 }
@@ -449,16 +501,20 @@ const int maildirFolder::getMessageCount()
 
 ref <folder> maildirFolder::getFolder(const folder::path::component& name)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 
-	return vmime::create <maildirFolder>(m_path / name, m_store);
+	return vmime::create <maildirFolder>(m_path / name, store);
 }
 
 
 std::vector <ref <folder> > maildirFolder::getFolders(const bool recursive)
 {
-	if (!isOpen() && !m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!isOpen() && !store)
 		throw exceptions::illegal_state("Store disconnected");
 
 	std::vector <ref <folder> > list;
@@ -471,12 +527,14 @@ std::vector <ref <folder> > maildirFolder::getFolders(const bool recursive)
 
 void maildirFolder::listFolders(std::vector <ref <folder> >& list, const bool recursive)
 {
+	ref <maildirStore> store = m_store.acquire();
+
 	try
 	{
-		utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+		utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 		ref <utility::file> rootDir = fsf->create
-			(maildirUtils::getFolderFSPath(m_store, m_path,
+			(maildirUtils::getFolderFSPath(store, m_path,
 				m_path.isEmpty() ? maildirUtils::FOLDER_PATH_ROOT
 				                 : maildirUtils::FOLDER_PATH_CONTAINER));
 
@@ -494,7 +552,7 @@ void maildirFolder::listFolders(std::vector <ref <folder> >& list, const bool re
 						m_path / file->getFullPath().getLastComponent();
 
 					ref <maildirFolder> subFolder =
-						vmime::create <maildirFolder>(subPath, m_store);
+						vmime::create <maildirFolder>(subPath, store);
 
 					list.push_back(subFolder);
 
@@ -517,27 +575,29 @@ void maildirFolder::listFolders(std::vector <ref <folder> >& list, const bool re
 
 void maildirFolder::rename(const folder::path& newPath)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (m_path.isEmpty() || newPath.isEmpty())
 		throw exceptions::illegal_operation("Cannot rename root folder");
-	else if (!m_store->isValidFolderName(newPath.getLastComponent()))
+	else if (!store->isValidFolderName(newPath.getLastComponent()))
 		throw exceptions::invalid_folder_name();
 
 	// Rename the directory on the file system
-	utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+	utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 	ref <utility::file> rootDir = fsf->create
-		(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_ROOT));
+		(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_ROOT));
 	ref <utility::file> contDir = fsf->create
-		(maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_CONTAINER));
+		(maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_CONTAINER));
 
 	try
 	{
 		const utility::file::path newRootPath =
-			maildirUtils::getFolderFSPath(m_store, newPath, maildirUtils::FOLDER_PATH_ROOT);
+			maildirUtils::getFolderFSPath(store, newPath, maildirUtils::FOLDER_PATH_ROOT);
 		const utility::file::path newContPath =
-			maildirUtils::getFolderFSPath(m_store, newPath, maildirUtils::FOLDER_PATH_CONTAINER);
+			maildirUtils::getFolderFSPath(store, newPath, maildirUtils::FOLDER_PATH_CONTAINER);
 
 		rootDir->rename(newRootPath);
 
@@ -555,9 +615,9 @@ void maildirFolder::rename(const folder::path& newPath)
 	{
 		// Revert to old location
 		const utility::file::path rootPath =
-			maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_ROOT);
+			maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_ROOT);
 		const utility::file::path contPath =
-			maildirUtils::getFolderFSPath(m_store, m_path, maildirUtils::FOLDER_PATH_CONTAINER);
+			maildirUtils::getFolderFSPath(store, m_path, maildirUtils::FOLDER_PATH_CONTAINER);
 
 		try
 		{
@@ -585,8 +645,8 @@ void maildirFolder::rename(const folder::path& newPath)
 	notifyFolder(event);
 
 	// Notify folders with the same path
-	for (std::list <maildirFolder*>::iterator it = m_store->m_folders.begin() ;
-	     it != m_store->m_folders.end() ; ++it)
+	for (std::list <maildirFolder*>::iterator it = store->m_folders.begin() ;
+	     it != store->m_folders.end() ; ++it)
 	{
 		if ((*it) != this && (*it)->getFullPath() == oldPath)
 		{
@@ -618,31 +678,33 @@ void maildirFolder::rename(const folder::path& newPath)
 void maildirFolder::deleteMessage(const int num)
 {
 	// Mark messages as deleted
-	setMessageFlags(num, num, message::FLAG_MODE_ADD, message::FLAG_DELETED);
+	setMessageFlags(num, num, message::FLAG_DELETED, message::FLAG_MODE_ADD);
 }
 
 
 void maildirFolder::deleteMessages(const int from, const int to)
 {
 	// Mark messages as deleted
-	setMessageFlags(from, to, message::FLAG_MODE_ADD, message::FLAG_DELETED);
+	setMessageFlags(from, to, message::FLAG_DELETED, message::FLAG_MODE_ADD);
 }
 
 
 void maildirFolder::deleteMessages(const std::vector <int>& nums)
 {
 	// Mark messages as deleted
-	setMessageFlags(nums, message::FLAG_MODE_ADD, message::FLAG_DELETED);
+	setMessageFlags(nums, message::FLAG_DELETED, message::FLAG_MODE_ADD);
 }
 
 
 void maildirFolder::setMessageFlags
 	(const int from, const int to, const int flags, const int mode)
 {
+	ref <maildirStore> store = m_store.acquire();
+
 	if (from < 1 || (to < from && to != -1))
 		throw exceptions::invalid_argument();
 
-	if (!m_store)
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (!isOpen())
 		throw exceptions::illegal_state("Folder not open");
@@ -725,7 +787,9 @@ void maildirFolder::setMessageFlags
 void maildirFolder::setMessageFlags
 	(const std::vector <int>& nums, const int flags, const int mode)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (!isOpen())
 		throw exceptions::illegal_state("Folder not open");
@@ -806,10 +870,12 @@ void maildirFolder::setMessageFlags
 void maildirFolder::setMessageFlagsImpl
 	(const std::vector <int>& nums, const int flags, const int mode)
 {
-	utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+	ref <maildirStore> store = m_store.acquire();
+
+	utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 	utility::file::path curDirPath = maildirUtils::getFolderFSPath
-		(m_store, m_path, maildirUtils::FOLDER_PATH_CUR);
+		(store, m_path, maildirUtils::FOLDER_PATH_CUR);
 
 	for (std::vector <int>::const_iterator it =
 	     nums.begin() ; it != nums.end() ; ++it)
@@ -869,19 +935,21 @@ void maildirFolder::addMessage(ref <vmime::message> msg, const int flags,
 void maildirFolder::addMessage(utility::inputStream& is, const int size,
 	const int flags, vmime::datetime* /* date */, utility::progressListener* progress)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (!isOpen())
 		throw exceptions::illegal_state("Folder not open");
 	else if (m_mode == MODE_READ_ONLY)
 		throw exceptions::illegal_state("Folder is read-only");
 
-	utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+	utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 	utility::file::path tmpDirPath = maildirUtils::getFolderFSPath
-		(m_store, m_path, maildirUtils::FOLDER_PATH_TMP);
+		(store, m_path, maildirUtils::FOLDER_PATH_TMP);
 	utility::file::path curDirPath = maildirUtils::getFolderFSPath
-		(m_store, m_path, maildirUtils::FOLDER_PATH_CUR);
+		(store, m_path, maildirUtils::FOLDER_PATH_CUR);
 
 	const utility::file::path::component filename =
 		maildirUtils::buildFilename(maildirUtils::generateId(),
@@ -932,8 +1000,8 @@ void maildirFolder::addMessage(utility::inputStream& is, const int size,
 	notifyMessageCount(event);
 
 	// Notify folders with the same path
-	for (std::list <maildirFolder*>::iterator it = m_store->m_folders.begin() ;
-	     it != m_store->m_folders.end() ; ++it)
+	for (std::list <maildirFolder*>::iterator it = store->m_folders.begin() ;
+	     it != store->m_folders.end() ; ++it)
 	{
 		if ((*it) != this && (*it)->getFullPath() == m_path)
 		{
@@ -958,7 +1026,7 @@ void maildirFolder::copyMessageImpl(const utility::file::path& tmpDirPath,
 	utility::inputStream& is, const utility::stream::size_type size,
 	utility::progressListener* progress)
 {
-	utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+	utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 	ref <utility::file> file = fsf->create(tmpDirPath / filename);
 
@@ -1040,7 +1108,9 @@ void maildirFolder::copyMessageImpl(const utility::file::path& tmpDirPath,
 
 void maildirFolder::copyMessage(const folder::path& dest, const int num)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (!isOpen())
 		throw exceptions::illegal_state("Folder not open");
@@ -1051,7 +1121,9 @@ void maildirFolder::copyMessage(const folder::path& dest, const int num)
 
 void maildirFolder::copyMessages(const folder::path& dest, const int from, const int to)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (!isOpen())
 		throw exceptions::illegal_state("Folder not open");
@@ -1075,7 +1147,9 @@ void maildirFolder::copyMessages(const folder::path& dest, const int from, const
 
 void maildirFolder::copyMessages(const folder::path& dest, const std::vector <int>& nums)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (!isOpen())
 		throw exceptions::illegal_state("Folder not open");
@@ -1087,15 +1161,17 @@ void maildirFolder::copyMessages(const folder::path& dest, const std::vector <in
 
 void maildirFolder::copyMessagesImpl(const folder::path& dest, const std::vector <int>& nums)
 {
-	utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+	ref <maildirStore> store = m_store.acquire();
+
+	utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 	utility::file::path curDirPath = maildirUtils::getFolderFSPath
-		(m_store, m_path, maildirUtils::FOLDER_PATH_CUR);
+		(store, m_path, maildirUtils::FOLDER_PATH_CUR);
 
 	utility::file::path destCurDirPath = maildirUtils::getFolderFSPath
-		(m_store, dest, maildirUtils::FOLDER_PATH_CUR);
+		(store, dest, maildirUtils::FOLDER_PATH_CUR);
 	utility::file::path destTmpDirPath = maildirUtils::getFolderFSPath
-		(m_store, dest, maildirUtils::FOLDER_PATH_TMP);
+		(store, dest, maildirUtils::FOLDER_PATH_TMP);
 
 	// Create destination directories
 	try
@@ -1151,8 +1227,10 @@ void maildirFolder::copyMessagesImpl(const folder::path& dest, const std::vector
 
 void maildirFolder::notifyMessagesCopied(const folder::path& dest)
 {
-	for (std::list <maildirFolder*>::iterator it = m_store->m_folders.begin() ;
-	     it != m_store->m_folders.end() ; ++it)
+	ref <maildirStore> store = m_store.acquire();
+
+	for (std::list <maildirFolder*>::iterator it = store->m_folders.begin() ;
+	     it != store->m_folders.end() ; ++it)
 	{
 		if ((*it) != this && (*it)->getFullPath() == dest)
 		{
@@ -1169,6 +1247,8 @@ void maildirFolder::notifyMessagesCopied(const folder::path& dest)
 
 void maildirFolder::status(int& count, int& unseen)
 {
+	ref <maildirStore> store = m_store.acquire();
+
 	const int oldCount = m_messageCount;
 
 	scanFolder();
@@ -1192,8 +1272,8 @@ void maildirFolder::status(int& count, int& unseen)
 		notifyMessageCount(event);
 
 		// Notify folders with the same path
-		for (std::list <maildirFolder*>::iterator it = m_store->m_folders.begin() ;
-		     it != m_store->m_folders.end() ; ++it)
+		for (std::list <maildirFolder*>::iterator it = store->m_folders.begin() ;
+		     it != store->m_folders.end() ; ++it)
 		{
 			if ((*it) != this && (*it)->getFullPath() == m_path)
 			{
@@ -1216,17 +1296,19 @@ void maildirFolder::status(int& count, int& unseen)
 
 void maildirFolder::expunge()
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (!isOpen())
 		throw exceptions::illegal_state("Folder not open");
 	else if (m_mode == MODE_READ_ONLY)
 		throw exceptions::illegal_state("Folder is read-only");
 
-	utility::fileSystemFactory* fsf = platformDependant::getHandler()->getFileSystemFactory();
+	utility::fileSystemFactory* fsf = platform::getHandler()->getFileSystemFactory();
 
 	utility::file::path curDirPath = maildirUtils::getFolderFSPath
-		(m_store, m_path, maildirUtils::FOLDER_PATH_CUR);
+		(store, m_path, maildirUtils::FOLDER_PATH_CUR);
 
 	std::vector <int> nums;
 	int unreadCount = 0;
@@ -1281,8 +1363,8 @@ void maildirFolder::expunge()
 	notifyMessageCount(event);
 
 	// Notify folders with the same path
-	for (std::list <maildirFolder*>::iterator it = m_store->m_folders.begin() ;
-	     it != m_store->m_folders.end() ; ++it)
+	for (std::list <maildirFolder*>::iterator it = store->m_folders.begin() ;
+	     it != store->m_folders.end() ; ++it)
 	{
 		if ((*it) != this && (*it)->getFullPath() == m_path)
 		{
@@ -1307,26 +1389,28 @@ ref <folder> maildirFolder::getParent()
 	if (m_path.isEmpty())
 		return NULL;
 	else
-		return vmime::create <maildirFolder>(m_path.getParent(), m_store);
+		return vmime::create <maildirFolder>(m_path.getParent(), m_store.acquire());
 }
 
 
-weak_ref <const store> maildirFolder::getStore() const
+ref <const store> maildirFolder::getStore() const
 {
-	return (m_store);
+	return m_store.acquire();
 }
 
 
-weak_ref <store> maildirFolder::getStore()
+ref <store> maildirFolder::getStore()
 {
-	return (m_store);
+	return m_store.acquire();
 }
 
 
 void maildirFolder::fetchMessages(std::vector <ref <message> >& msg,
 	const int options, utility::progressListener* progress)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (!isOpen())
 		throw exceptions::illegal_state("Folder not open");
@@ -1337,12 +1421,12 @@ void maildirFolder::fetchMessages(std::vector <ref <message> >& msg,
 	if (progress)
 		progress->start(total);
 
-	weak_ref <maildirFolder> _this = thisWeakRef().dynamicCast <maildirFolder>();
+	ref <maildirFolder> thisFolder = thisRef().dynamicCast <maildirFolder>();
 
 	for (std::vector <ref <message> >::iterator it = msg.begin() ;
 	     it != msg.end() ; ++it)
 	{
-		(*it).dynamicCast <maildirMessage>()->fetch(_this, options);
+		(*it).dynamicCast <maildirMessage>()->fetch(thisFolder, options);
 
 		if (progress)
 			progress->progress(++current, total);
@@ -1355,13 +1439,15 @@ void maildirFolder::fetchMessages(std::vector <ref <message> >& msg,
 
 void maildirFolder::fetchMessage(ref <message> msg, const int options)
 {
-	if (!m_store)
+	ref <maildirStore> store = m_store.acquire();
+
+	if (!store)
 		throw exceptions::illegal_state("Store disconnected");
 	else if (!isOpen())
 		throw exceptions::illegal_state("Folder not open");
 
 	msg.dynamicCast <maildirMessage>()->fetch
-		(thisWeakRef().dynamicCast <maildirFolder>(), options);
+		(thisRef().dynamicCast <maildirFolder>(), options);
 }
 
 
@@ -1376,7 +1462,7 @@ const int maildirFolder::getFetchCapabilities() const
 const utility::file::path maildirFolder::getMessageFSPath(const int number) const
 {
 	utility::file::path curDirPath = maildirUtils::getFolderFSPath
-		(m_store, m_path, maildirUtils::FOLDER_PATH_CUR);
+		(m_store.acquire(), m_path, maildirUtils::FOLDER_PATH_CUR);
 
 	return (curDirPath / m_messageInfos[number - 1].path);
 }
