@@ -1,6 +1,6 @@
 //
 // VMime library (http://www.vmime.org)
-// Copyright (C) 2002-2005 Vincent Richard <vincent@vincent-richard.net>
+// Copyright (C) 2002-2006 Vincent Richard <vincent@vincent-richard.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -51,6 +51,8 @@ private:
 public:
 
 	exception(const string& what, const exception& other = NO_EXCEPTION);
+	exception(const exception& e);
+
 	virtual ~exception() throw();
 
 	/** Return a description of the error.
@@ -127,7 +129,7 @@ class no_encoder_available : public vmime::exception
 {
 public:
 
-	no_encoder_available(const exception& other = NO_EXCEPTION);
+	no_encoder_available(const string& name, const exception& other = NO_EXCEPTION);
 	~no_encoder_available() throw();
 
 	exception* clone() const;
@@ -246,12 +248,12 @@ public:
 };
 
 
-class no_platform_dependant_handler : public vmime::exception
+class no_platform_handler : public vmime::exception
 {
 public:
 
-	no_platform_dependant_handler(const exception& other = NO_EXCEPTION);
-	~no_platform_dependant_handler() throw();
+	no_platform_handler(const exception& other = NO_EXCEPTION);
+	~no_platform_handler() throw();
 
 	exception* clone() const;
 	const char* name() const throw();
@@ -653,16 +655,16 @@ public:
 	~command_error() throw();
 
 	/** Return the name of the command which have thrown the exception.
-	  * This is protocol-dependant.
+	  * This is protocol-dependent.
 	  *
-	  * @return command name (protocol-dependant)
+	  * @return command name (protocol-dependent)
 	  */
 	const string& command() const;
 
 	/** Return the invalid response line.
-	  * The meaning is protocol-dependant.
+	  * The meaning is protocol-dependent.
 	  *
-	  * @return response line (protocol-dependant)
+	  * @return response line (protocol-dependent)
 	  */
 	const string& response() const;
 
@@ -687,16 +689,16 @@ public:
 	~invalid_response() throw();
 
 	/** Return the name of the command which have thrown the exception.
-	  * This is protocol-dependant.
+	  * This is protocol-dependent.
 	  *
-	  * @return command name (protocol-dependant)
+	  * @return command name (protocol-dependent)
 	  */
 	const string& command() const;
 
 	/** Return the invalid response line.
-	  * The meaning is protocol-dependant.
+	  * The meaning is protocol-dependent.
 	  *
-	  * @return response line (protocol-dependant)
+	  * @return response line (protocol-dependent)
 	  */
 	const string& response() const;
 
